@@ -1,5 +1,6 @@
+
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2001 Damien Sandras
+ * Copyright (C) 2000-2004 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,21 +15,33 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *
+ *
+ * GnomeMeting is licensed under the GPL license and as a special exception,
+ * you have permission to link or otherwise combine this program with the
+ * programs OpenH323 and Pwlib, and distribute the combination, without
+ * applying the requirements of the GNU GPL to the OpenH323 program, as long
+ * as you do follow the requirements of the GNU GPL for all the rest of the
+ * software thus combined.
  */
 
+
 /*
  *                         druid.h  -  description
  *                         -----------------------
  *   begin                : Mon May 1 2002
- *   copyright            : (C) 2000-2002 by Damien Sandras
+ *   copyright            : (C) 2000-2004 by Damien Sandras
  *   description          : This file contains all the functions needed to
  *                          build the druid.
- *   email                : dsandras@seconix.com
  */
 
+#include "common.h"
+
 
 /* DESCRIPTION  :  /
- * BEHAVIOR     :  Builds the "First time configuration druid".
+ * BEHAVIOR     :  Builds the "First time configuration druid" and returns it.
  * PRE          :  /
  */
-void gnomemeeting_init_druid (gpointer data);
+GtkWidget *
+gnomemeeting_druid_window_new (GmDruidWindow *);
+
