@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2003 Damien Sandras
+ * Copyright (C) 2000-2004 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
  *                         endpoint.cpp  -  description
  *                         ----------------------------
  *   begin                : Sat Dec 23 2000
- *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   copyright            : (C) 2000-2004 by Damien Sandras
  *   description          : This file contains the Endpoint class.
  *
  */
@@ -93,7 +93,7 @@ GMH323EndPoint::GMH323EndPoint ()
 #endif
   ils_client = NULL;
   listener = NULL;
-
+  gk = NULL;
 
   /* Use IPv6 address family by default if available. */
 #ifdef P_HAS_IPV6
@@ -101,8 +101,6 @@ GMH323EndPoint::GMH323EndPoint ()
     PIPSocket::SetDefaultIpAddressFamilyV6();
 #endif
   
-  received_video_device = NULL;
-  transmitted_video_device = NULL;
   audio_tester = NULL;
   
   ILSTimer.SetNotifier (PCREATE_NOTIFIER (OnILSTimeout));
@@ -112,6 +110,8 @@ GMH323EndPoint::GMH323EndPoint ()
   GatewayIPTimer.SetNotifier (PCREATE_NOTIFIER (OnGatewayIPTimeout));
   GatewayIPTimer.RunContinuous (PTimeInterval (5));
 
+  signallingChannelCallTimeout = PTimeInterval (0, 0, 3);
+    
   NoIncomingMediaTimer.SetNotifier (PCREATE_NOTIFIER (OnNoIncomingMediaTimeout));
   NoAnswerTimer.SetNotifier (PCREATE_NOTIFIER (OnNoAnswerTimeout));
   CallPendingTimer.SetNotifier (PCREATE_NOTIFIER (OnCallPending));
@@ -124,6 +124,12 @@ GMH323EndPoint::~GMH323EndPoint ()
   if (listener)
     RemoveListener (listener);
 
+  /* Delete any GMH323Gatekeeper thread, the real gatekeeper
+   * will be delete by openh323 
+   */
+  if (gk)
+    delete (gk);
+  
   PWaitAndSignal m(ils_access_mutex);
   /* Delete any ILS client which could be running */
   if (ils_client)
@@ -418,18 +424,22 @@ GMH323EndPoint::AddAudioCapabilities ()
     SetCapability (0, 0, new SpeexNarrow3AudioCapability ());
 
     SetCapability (0, 0, gsm_capa = new MicrosoftGSMAudioCapability);
-    gsm_capa->SetTxFramesInPacket (gsm_frames);
+    if (gsm_frames > 0)
+      gsm_capa->SetTxFramesInPacket (gsm_frames);
 
     g711_capa = new H323_G711Capability (H323_G711Capability::muLaw);
     SetCapability (0, 0, g711_capa);
-    g711_capa->SetTxFramesInPacket (g711_frames);
+    if (g711_frames > 0)
+      g711_capa->SetTxFramesInPacket (g711_frames);
 
     g711_capa = new H323_G711Capability (H323_G711Capability::ALaw);
     SetCapability (0, 0, g711_capa);
-    g711_capa->SetTxFramesInPacket (g711_frames);
+    if (g711_frames > 0)
+      g711_capa->SetTxFramesInPacket (g711_frames);
 
     SetCapability (0, 0, gsm2_capa = new H323_GSM0610Capability);	
-    gsm2_capa->SetTxFramesInPacket (gsm_frames);
+    if (gsm_frames > 0)
+      gsm2_capa->SetTxFramesInPacket (gsm_frames);
 
     g72616_capa = 
       new H323_G726_Capability (*this, H323_G726_Capability::e_32k);
@@ -627,7 +637,7 @@ GMH323EndPoint::GetVideoGrabber ()
 H323Gatekeeper *
 GMH323EndPoint::CreateGatekeeper(H323Transport * transport)
 {
-  return new H323GatekeeperWithNAT(*this, transport);
+  return new H323GatekeeperWithNAT (*this, transport);
 }
 
 
@@ -674,7 +684,10 @@ GMH323EndPoint::GetGatekeeper ()
 void 
 GMH323EndPoint::GatekeeperRegister ()
 {
-  new GMH323Gatekeeper ();
+  if (gk)
+    delete (gk);
+  
+  gk = new GMH323Gatekeeper ();
 }
 
 
@@ -683,21 +696,27 @@ GMH323EndPoint::OnIncomingCall (H323Connection & connection,
                                 const H323SignalPDU &, H323SignalPDU &)
 {
   char *msg = NULL;
+  
+  PString gateway;
   PString forward_host;
 
+  int no_answer_timeout = 45;
   gchar *utf8_name = NULL;
   gchar *utf8_app = NULL;
   gchar *utf8_url = NULL;
   
   gchar *forward_host_gconf = NULL;
+  gchar *gateway_gconf = NULL;
+
   IncomingCallMode icm = AVAILABLE;
+  
   BOOL busy_forward = FALSE;
   BOOL show_popup = FALSE;
-
   BOOL do_forward = FALSE;
   BOOL do_reject = FALSE;
   BOOL do_answer = FALSE;
-
+  BOOL use_gateway = FALSE;
+  
 #ifdef HAS_IXJ
   GMLid *l = NULL;
 #endif    
@@ -709,29 +728,35 @@ GMH323EndPoint::OnIncomingCall (H323Connection & connection,
   icm =
     (IncomingCallMode) gconf_get_int (CALL_OPTIONS_KEY "incoming_call_mode");
   show_popup = gconf_get_bool (USER_INTERFACE_KEY "show_popup");
+  no_answer_timeout = gconf_get_int (CALL_OPTIONS_KEY "no_answer_timeout");
+  use_gateway = gconf_get_bool (H323_GATEWAY_KEY "use_gateway");
+  gateway_gconf = gconf_get_string (H323_GATEWAY_KEY "host");
   gnomemeeting_threads_leave ();
 
+
   if (forward_host_gconf)
     forward_host = PString (GMURL (forward_host_gconf).GetValidURL ());
   else
     forward_host = PString ("");
-
     
+  gateway = PString (gateway_gconf);
+
+
   /* Remote Name and application */
   GetRemoteConnectionInfo (connection, utf8_name, utf8_app, utf8_url);
 
 
-  /* Update the history and status bar */
+  /* Update the log and status bar */
   msg = g_strdup_printf (_("Call from %s"), (const char *) utf8_name);
   gnomemeeting_threads_enter ();
   gnomemeeting_statusbar_push (gw->statusbar, msg);
-  gnomemeeting_log_insert (gw->history_text_view, msg);
+  gnomemeeting_log_insert (msg);
   gnomemeeting_threads_leave ();
   g_free (msg);
 
 
   /* Check what action to take */
-  if (!forward_host.IsEmpty() && icm == FORWARD) {
+  if (!GMURL(forward_host).IsEmpty() && icm == FORWARD) {
 
     msg = 
       g_strdup_printf (_("Forwarding call from %s to %s (Forward all calls)"),
@@ -781,12 +806,13 @@ GMH323EndPoint::OnIncomingCall (H323Connection & connection,
   /* Take that action */
   if (do_reject || do_forward || do_answer) {
 
-    /* Add the full message in the history */
+    /* Add the full message in the log */
     gnomemeeting_threads_enter ();
-    gnomemeeting_log_insert (gw->history_text_view, msg);
+    gnomemeeting_log_insert (msg);
     gnomemeeting_threads_leave ();
 
     /* Free things, we will return */
+    g_free (gateway_gconf);
     g_free (forward_host_gconf);
     g_free (utf8_name);
     g_free (utf8_app);
@@ -807,6 +833,9 @@ GMH323EndPoint::OnIncomingCall (H323Connection & connection,
       gnomemeeting_statusbar_push (gw->statusbar, _("Call forwarded"));
       gnomemeeting_threads_leave ();
 
+      if (use_gateway && !gateway.IsEmpty ())
+        forward_host = forward_host + "@" + gateway;
+
       return !connection.ForwardCall (forward_host);
     }
     else if (do_answer) {
@@ -842,7 +871,7 @@ GMH323EndPoint::OnIncomingCall (H323Connection & connection,
 
     
   /* The timers */
-  NoAnswerTimer.SetInterval (0, 15);
+  NoAnswerTimer.SetInterval (0, PMAX (no_answer_timeout, 10));
   CallPendingTimer.RunContinuous (PTimeInterval (5));
 
     
@@ -860,6 +889,7 @@ GMH323EndPoint::OnIncomingCall (H323Connection & connection,
   SetCurrentCallToken (connection.GetCallToken ());
   SetCallingState (GMH323EndPoint::Called);
 
+  g_free (gateway_gconf);
   g_free (forward_host_gconf);
   g_free (utf8_name);
   g_free (utf8_app);
@@ -885,7 +915,7 @@ GMH323EndPoint::OnConnectionForwarded (H323Connection &,
     msg = g_strdup_printf (_("Forwarding call to %s"),
 			   (const char*) forward_party);
     gnomemeeting_statusbar_push (gw->statusbar, msg);
-    gnomemeeting_log_insert (gw->history_text_view, msg);
+    gnomemeeting_log_insert (msg);
     gnomemeeting_threads_leave ();
     g_free (msg);
 
@@ -916,13 +946,15 @@ GMH323EndPoint::OnConnectionEstablished (H323Connection & connection,
   gchar *utf8_app = NULL;
   gchar *utf8_name = NULL;
   gchar *utf8_local_name = NULL;
-  char *msg = NULL;
   BOOL reg = FALSE;
+  BOOL forward_on_busy = FALSE;
+  IncomingCallMode icm = AVAILABLE;
 
+  
 #ifdef HAS_IXJ
   GMLid *l = NULL;
 #endif
-
+  
   
   /* Start refreshing the stats */
   RTPTimer.RunContinuous (PTimeInterval (0, 1));
@@ -936,16 +968,19 @@ GMH323EndPoint::OnConnectionEstablished (H323Connection & connection,
   /* Get the gconf settings */
   gnomemeeting_threads_enter ();
   reg = gconf_get_bool (LDAP_KEY "enable_registering");
+  icm = 
+    (IncomingCallMode) gconf_get_int (CALL_OPTIONS_KEY "incoming_call_mode");
+  forward_on_busy = gconf_get_bool (CALL_FORWARDING_KEY "forward_on_busy");
   gnomemeeting_threads_leave ();
   
   
   /* Connected */
   gnomemeeting_threads_enter ();
 
-  msg = g_strdup_printf (_("Connected with %s using %s"), 
-			 utf8_name, utf8_app);
   gnomemeeting_statusbar_push (gw->statusbar, _("Connected"));
-  gnomemeeting_log_insert (gw->history_text_view, msg);
+  gnomemeeting_log_insert (_("Connected with %s using %s"), 
+			 utf8_name, utf8_app);
+  gnomemeeting_text_chat_call_start_notification ();
 
   gtk_label_set_text (GTK_LABEL (gw->remote_name), (const char *) utf8_name);
   gtk_window_set_title (GTK_WINDOW (gw->remote_video_window), 
@@ -1000,7 +1035,8 @@ GMH323EndPoint::OnConnectionEstablished (H323Connection & connection,
   gnomemeeting_addressbook_update_menu_sensitivity ();
   gnomemeeting_main_window_update_sensitivity (GMH323EndPoint::Connected);
   gnomemeeting_menu_update_sensitivity (GMH323EndPoint::Connected);
-  gnomemeeting_tray_set_content (gw->docklet, 2);
+  gnomemeeting_tray_update (gw->docklet, GMH323EndPoint::Connected, 
+                            icm, forward_on_busy);
   gnomemeeting_threads_leave ();
 
   
@@ -1008,7 +1044,6 @@ GMH323EndPoint::OnConnectionEstablished (H323Connection & connection,
   if (reg)
     ILSRegister ();
 
-  g_free (msg);
   g_free (utf8_name);
   g_free (utf8_local_name);
   g_free (utf8_app);
@@ -1086,6 +1121,9 @@ GMH323EndPoint::OnConnectionCleared (H323Connection & connection,
   BOOL reg = FALSE;
   BOOL stay_on_top = FALSE;
   BOOL not_current = FALSE;
+  BOOL forward_on_busy = FALSE;
+
+  IncomingCallMode icm = AVAILABLE;
 
   GmRtpData *rtp = NULL;
 
@@ -1105,6 +1143,10 @@ GMH323EndPoint::OnConnectionCleared (H323Connection & connection,
     gconf_get_bool (USER_INTERFACE_KEY "auto_clear_text_chat");
   reg = gconf_get_bool (LDAP_KEY "enable_registering");
   stay_on_top = gconf_get_bool (VIDEO_DISPLAY_KEY "stay_on_top");
+  icm = (IncomingCallMode)
+    gconf_get_int (CALL_OPTIONS_KEY "incoming_call_mode");
+  forward_on_busy =
+    gconf_get_bool (CALL_FORWARDING_KEY "forward_on_busy");
   gnomemeeting_threads_leave ();
 
 
@@ -1209,27 +1251,28 @@ GMH323EndPoint::OnConnectionCleared (H323Connection & connection,
 
   gnomemeeting_threads_enter ();
   if (t.GetSeconds () == 0 && connection.HadAnsweredCall ())
-    gnomemeeting_calls_history_window_add_call (2, utf8_name,
+    gnomemeeting_calls_history_window_add_call (MISSED_CALL, utf8_name,
 						utf8_url,
 						"0",
 						msg_reason,
 						utf8_app);
   else
     if (connection.HadAnsweredCall ())
-      gnomemeeting_calls_history_window_add_call (0, utf8_name,
+      gnomemeeting_calls_history_window_add_call (RECEIVED_CALL, utf8_name,
 						  utf8_url,
-						  t.AsString (2),
+						  t.AsString (0),
 						  msg_reason,
 						  utf8_app);
     else
-      gnomemeeting_calls_history_window_add_call (1, utf8_name,
+      gnomemeeting_calls_history_window_add_call (PLACED_CALL, utf8_name,
 						  GetLastCallAddress (),
-						  t.AsString (2),
+						  t.AsString (0),
 						  msg_reason,
 						  utf8_app);
 
 
-  gnomemeeting_log_insert (gw->history_text_view, msg_reason);
+  gnomemeeting_log_insert (msg_reason);
+  gnomemeeting_text_chat_call_stop_notification ();
   gnomemeeting_statusbar_flash (gw->statusbar, msg_reason);
   gnomemeeting_threads_leave ();
 
@@ -1250,6 +1293,7 @@ GMH323EndPoint::OnConnectionCleared (H323Connection & connection,
   NoIncomingMediaTimer.Stop ();
   
   
+  gnomemeeting_threads_enter ();
   if (gw->incoming_call_popup) {
 
     gtk_widget_destroy (gw->incoming_call_popup);
@@ -1313,10 +1357,12 @@ GMH323EndPoint::OnConnectionCleared (H323Connection & connection,
   gw->audio_transmission_popup = NULL;
 
   
-  /* No Audio reception or transmission */
+  /* We update the GUI */
   gnomemeeting_menu_update_sensitivity (GMH323EndPoint::Standby);
   gnomemeeting_main_window_update_sensitivity (GMH323EndPoint::Standby);
   gnomemeeting_addressbook_update_menu_sensitivity ();
+  gnomemeeting_tray_update (gw->docklet, GMH323EndPoint::Standby, 
+                            icm, forward_on_busy);
   gnomemeeting_threads_leave ();
   
   gnomemeeting_sound_daemons_resume ();
@@ -1489,7 +1535,7 @@ GMH323EndPoint::Init ()
   disableH245Tunneling = !gconf_get_bool (H323_ADVANCED_KEY "enable_h245_tunneling");
   disableFastStart = !gconf_get_bool (H323_ADVANCED_KEY "enable_fast_start");
   disableH245inSetup = !gconf_get_bool (H323_ADVANCED_KEY "enable_early_h245");
-
+    
   /* Setup ports */
   SetPorts ();
   
@@ -1589,15 +1635,13 @@ GMH323EndPoint::OpenAudioChannel (H323Connection & connection,
   PString plugin;
   PString device;
   
-  unsigned int vol_rec = 0;
-  unsigned int vol_play = 0;
+  unsigned int vol = 0;
 
   BOOL sd = FALSE;
   BOOL no_error = TRUE;
-  
-  gchar *msg = NULL;
 
-      
+  PWaitAndSignal m(audio_channel_mutex);
+
   /* Wait that the primary call has terminated (in case of transfer)
      before opening the channels for the second call */
   TransferCallWait ();
@@ -1629,10 +1673,8 @@ GMH323EndPoint::OpenAudioChannel (H323Connection & connection,
   if ((l = GetLid ()) && l->IsOpen()) {
 
     lid->StopTone (0);
-    
-    if (!codec.AttachChannel (new OpalLineChannel (*l,
-						   OpalIxJDevice::POTSLine,
-						   codec))) {
+      
+    if (!codec.AttachChannel (new OpalLineChannel (*l, OpalIxJDevice::POTSLine, codec))) {
 
       l->Unlock ();
 
@@ -1643,9 +1685,7 @@ GMH323EndPoint::OpenAudioChannel (H323Connection & connection,
 	l->Unlock ();
 
     gnomemeeting_threads_enter ();
-    gchar *msg = g_strdup_printf (_("Attaching lid hardware to codec"));
-    gnomemeeting_log_insert (gw->history_text_view, msg);
-    g_free (msg);    
+    gnomemeeting_log_insert (_("Attaching lid hardware to codec"));
     gnomemeeting_threads_leave ();
   }
   else
@@ -1676,15 +1716,14 @@ GMH323EndPoint::OpenAudioChannel (H323Connection & connection,
 
 	  /* Translators : the full sentence is "Opening %s for playing with
 	     plugin %s" or "Opening %s for recording with plugin" */
-	  msg = g_strdup_printf (_("Opened %s for %s with plugin %s"),
-				 (const char *) device,
-				 is_encoding ? _("recording") : _("playing"),
-				 (const char *) plugin);
 
 	  gnomemeeting_threads_enter ();
-	  gnomemeeting_log_insert (gw->history_text_view, msg);
+	  gnomemeeting_log_insert (is_encoding ?
+				   _("Opened %s for recording with plugin %s")
+				   : _("Opened %s for playing with plugin %s"),
+				   (const char *) device, 
+                                   (const char *) plugin);
 	  gnomemeeting_threads_leave ();
-	  g_free (msg);
 
 	  /* Control the channel and attach it to the codec, do not
 	     auto delete it */
@@ -1693,17 +1732,21 @@ GMH323EndPoint::OpenAudioChannel (H323Connection & connection,
 
 	  /* Update the volume sliders */
 	  gnomemeeting_threads_enter ();
-	  GetDeviceVolume (vol_play, vol_rec);
-	  GTK_ADJUSTMENT (gw->adj_play)->value = vol_play;
-	  GTK_ADJUSTMENT (gw->adj_rec)->value = vol_rec;
+	  GetDeviceVolume (sound_channel, is_encoding, vol);
+          if (!is_encoding)
+            GTK_ADJUSTMENT (gw->adj_play)->value = vol;
+          else
+            GTK_ADJUSTMENT (gw->adj_rec)->value = vol;
 	  gtk_widget_queue_draw (GTK_WIDGET (gw->audio_settings_frame));
 	  gnomemeeting_threads_leave ();
 	}
 	else
 	  no_error = FALSE; /* No PSoundChannel */
       }
-      else
+      else {
+
 	return FALSE; /* Device was _("No device found"), ignore, no popup */
+      }
     }
 
   
@@ -1723,25 +1766,26 @@ GMH323EndPoint::OpenAudioChannel (H323Connection & connection,
 	  gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Could not open audio channel for audio reception"), _("An error occured while trying to play audio to the soundcard for the audio reception. Please check that your soundcard is not busy and that your driver supports full-duplex.\nThe audio reception has been disabled."));
     gnomemeeting_threads_leave ();
   }
-
     
   return no_error;
 }
 
 
 BOOL
-GMH323EndPoint::SetDeviceVolume (unsigned int play_vol, 
-				 unsigned int rec_vol)
+GMH323EndPoint::SetDeviceVolume (PSoundChannel *sound_channel,
+                                 BOOL is_encoding,
+                                 unsigned int vol)
 {
-  return DeviceVolume (TRUE, play_vol, rec_vol);
+  return DeviceVolume (sound_channel, is_encoding, TRUE, vol);
 }
 
 
 BOOL
-GMH323EndPoint::GetDeviceVolume (unsigned int &play_vol, 
-				 unsigned int &rec_vol)
+GMH323EndPoint::GetDeviceVolume (PSoundChannel *sound_channel,
+                                 BOOL is_encoding,
+                                 unsigned int &vol)
 {
-  return DeviceVolume (FALSE, play_vol, rec_vol);
+  return DeviceVolume (sound_channel, is_encoding, FALSE, vol);
 }
 
 
@@ -1774,34 +1818,14 @@ GMH323EndPoint::StartLogicalChannel (const PString & capability_name,
 	  !con->OpenLogicalChannel (*capability,
 				    capability->GetDefaultSessionID(),
 				    H323Channel::IsTransmitter)) {
-	no_error = FALSE;
+        no_error = FALSE;
       }
     }
 
-    /** This code is not supported yet (Mode Changes).
-    else {
-
-      if (id == RTP_Session::DefaultVideoSessionID)
-	channel = con->FindChannel (RTP_Session::DefaultAudioSessionID, TRUE);
-      else
-	channel = con->FindChannel (RTP_Session::DefaultVideoSessionID, TRUE);
-
-      if (channel) {
-
-	current_capa = channel->GetCapability ().GetFormatName ();
-
-	if (!current_capa.IsEmpty ())
-	  mode = capability_name + "\t" + current_capa;
-
-	con->RequestModeChange (mode);
-      }
-      else
-	no_error = TRUE;
-    } */
-
     con->Unlock ();
   }
 
+    
   return no_error;
 }
 
@@ -1821,8 +1845,10 @@ GMH323EndPoint::StopLogicalChannel (unsigned id,
     channel =
       con->FindChannel (id, from_remote);
 
-    if (channel) 	  
+    if (channel) {
+      
       con->CloseLogicalChannelNumber (channel->GetNumber ());
+    }
     else 
       no_error = FALSE;
 
@@ -2036,7 +2062,8 @@ GMH323EndPoint::OnGatewayIPTimeout (PTimer &,
 
   if (ip_checking) {
 
-    if (web_client.GetTextDocument ("http://213.193.144.104/ip/", html)) {
+    gchar *ip_detector = gconf_get_string (NAT_KEY "public_ip_detector");
+    if (ip_detector != NULL && web_client.GetTextDocument (ip_detector, html)) {
 
       if (!html.IsEmpty ()) {
 
@@ -2048,7 +2075,8 @@ GMH323EndPoint::OnGatewayIPTimeout (PTimer &,
 
       }
     }
-
+    if (ip_detector != NULL)
+      g_free (ip_detector);
     if (!ip_address.IsEmpty ()) {
 
       gdk_threads_enter ();
@@ -2070,7 +2098,6 @@ GMH323EndPoint::OnNoAnswerTimeout (PTimer &,
  
   H323Connection *connection = NULL;
   
-  gchar *msg = NULL;
   gchar *forward_host_gconf = NULL;
   
   PString forward_host;
@@ -2112,12 +2139,10 @@ GMH323EndPoint::OnNoAnswerTimeout (PTimer &,
       connection->ForwardCall (PString ((const char *) forward_host));
 
       gdk_threads_enter ();
-      msg = g_strdup_printf (_("Forwarding Call to %s (No Answer)"), 
-			     (const char *) forward_host);
-      gnomemeeting_log_insert (gw->history_text_view, msg);
+      gnomemeeting_log_insert (_("Forwarding Call to %s (No Answer)"), 
+				(const char *) forward_host);
 
       gnomemeeting_statusbar_push (gw->statusbar, _("Call forwarded"));
-      g_free (msg);
       gdk_threads_leave ();
 
       connection->Unlock ();
@@ -2143,7 +2168,7 @@ GMH323EndPoint::OnCallPending (PTimer &,
   gw = GnomeMeeting::Process ()->GetMainWindow ();
 
   gdk_threads_enter ();
-  gnomemeeting_tray_flash (gw->docklet);
+  gnomemeeting_tray_ring (gw->docklet);
   is_ringing = gnomemeeting_tray_is_ringing (gw->docklet);
   gdk_threads_leave ();
 
@@ -2174,18 +2199,15 @@ GMH323EndPoint::OnOutgoingCall (PTimer &,
 
 
 BOOL 
-GMH323EndPoint::DeviceVolume (BOOL set, 
-			      unsigned int &play_vol, 
-			      unsigned int &rec_vol)
+GMH323EndPoint::DeviceVolume (PSoundChannel *sound_channel,
+                              BOOL is_encoding,
+                              BOOL set, 
+			      unsigned int &vol) 
 {
-  H323Channel *channel = NULL;
-  PSoundChannel *sound_channel = NULL;
-  H323Codec *raw_codec = NULL;
   H323Connection *con = NULL;
 
 #ifdef HAS_IXJ
   GMLid *l = NULL;
-  BOOL has_lid = FALSE;
 #endif
 
   BOOL err = TRUE;
@@ -2199,49 +2221,39 @@ GMH323EndPoint::DeviceVolume (BOOL set,
 
 #ifdef HAS_IXJ
     if (set) {
-      
+
       l = GetLid ();
       if (l) {
 
-	l->SetVolume (play_vol, rec_vol);
-	l->Unlock ();
-	has_lid = TRUE;
+        l->SetVolume (is_encoding, vol);
+        l->Unlock ();
+
+        con->Unlock ();
+
+        return TRUE; /* Ignore the sound channel if a LID is used. Hopefully,
+                        OpenH323 will be fixed soon */
       }
     }
 #endif
 
-    if (!has_lid) {
 
-      for (int cpt = 0 ; cpt < 2 ; cpt ++) {
+    /* TRUE = from_remote = playing */
+    if (sound_channel) {
 
-	/* TRUE = from_remote = playing */
-	channel = 
-	  con->FindChannel (RTP_Session::DefaultAudioSessionID, (cpt == 0));
+      if (set) {
 
-	if (channel) {
+        err = 
+          sound_channel->SetVolume (vol)
+          && err;
+      }
+      else {
 
-	  raw_codec = channel->GetCodec();
-
-	  if (raw_codec) {
-
-	    sound_channel = (PSoundChannel *) raw_codec->GetRawDataChannel ();
-	    if (sound_channel)
-	      if (set) {
-	      
-		err = 
-		  sound_channel->SetVolume ((cpt == 0)?play_vol:rec_vol)
-		  && err;
-	      }
-	      else {
-
-		err = 
-		  sound_channel->GetVolume ((cpt == 0)?play_vol:rec_vol)
-		  && err;
-	      }
-	  }
-	}
+        err = 
+          sound_channel->GetVolume (vol)
+          && err;
       }
     }
+
     con->Unlock ();
   }
 
@@ -2261,10 +2273,13 @@ GMH323EndPoint::OpenVideoChannel (H323Connection & connection,
   double frame_time = 0.0;
 
   PVideoChannel *channel = NULL;
+  GDKVideoOutputDevice *display_device = NULL;
+  
   GMVideoGrabber *vg = NULL;
 
   BOOL result = FALSE;
 
+  PWaitAndSignal m(video_channel_mutex);
   
   /* Wait that the primary call has terminated (in case of transfer)
      before opening the channels for the second call */
@@ -2275,7 +2290,6 @@ GMH323EndPoint::OpenVideoChannel (H323Connection & connection,
   NoAnswerTimer.Stop ();
   CallPendingTimer.Stop ();
 
-  
   /* Transmitting */
   if (is_encoding && autoStartTransmitVideo) {
 
@@ -2304,51 +2318,48 @@ GMH323EndPoint::OpenVideoChannel (H323Connection & connection,
 			H323VideoCodec::AdaptivePacketDelay |
 			codec.GetVideoMode());
 
-
-    /* Make sure there is a grabber */    
+    /* Needed to be able to stop start the channel on-the-fly. When
+     * the channel has been closed, the rawdata channel has been closed
+     * too but not deleted. We delete it now.
+     */
     vg = GetVideoGrabber ();
-    if (!vg) {
-      
+    if (!vg || !vg->IsChannelOpen ()) {
+
       CreateVideoGrabber (FALSE, TRUE);
       vg = GetVideoGrabber ();
     }
-    
+      
     if (vg) {
-
+    
+      vg->StopGrabbing ();
       channel = vg->GetVideoChannel ();
-      transmitted_video_device = vg->GetEncodingDevice ();
       vg->Unlock ();
     }
-      
+    else
+      return FALSE;
 
-    gnomemeeting_threads_enter ();
-    gtk_widget_set_sensitive (GTK_WIDGET (gw->video_chan_button), TRUE);
-    gnomemeeting_threads_leave ();
 
     if (channel)
       result = codec.AttachChannel (channel, FALSE);
-    
+   
     return result;
   }
   else if (!is_encoding && autoStartReceiveVideo) {
 
     channel = new PVideoChannel;
-    received_video_device = new GDKVideoOutputDevice (is_encoding, gw);
-    received_video_device->SetColourFormatConverter ("YUV420P");      
-    channel->AttachVideoPlayer (received_video_device);
-     
-    /* Stop to grab */
+    display_device = new GDKVideoOutputDevice (is_encoding, gw);
+    display_device->SetColourFormatConverter ("YUV420P");      
+    channel->AttachVideoPlayer (display_device);
+
     vg = GetVideoGrabber ();
     if (vg) {
-
-      if (autoStartTransmitVideo) {
-	  
-	vg->StopGrabbing ();
-	vg->Unlock ();
-      }
-      else
-	RemoveVideoGrabber ();
+      
+      if (!autoStartTransmitVideo) 
+        vg->StopGrabbing ();
+      
+      vg->Unlock ();
     }
+      
 
     if (channel)
       result = codec.AttachChannel (channel);
@@ -2356,7 +2367,7 @@ GMH323EndPoint::OpenVideoChannel (H323Connection & connection,
     return result;
   }
 
-  
+
   return FALSE;
 }
 
@@ -2439,3 +2450,4 @@ GMH323EndPoint::TransferCallWait ()
   while (!GetTransferCallToken ().IsEmpty ())
     PThread::Current ()->Sleep (100);
 }
+          
