@@ -1,5 +1,5 @@
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2003 Damien Sandras
+ * Copyright (C) 2000-2004 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
  *                         gnomemeeting.cpp  -  description
  *                         --------------------------------
  *   begin                : Sat Dec 23 2000
- *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   copyright            : (C) 2000-2004 by Damien Sandras
  *   description          : This file contains the main class
  *
  */
@@ -81,7 +81,7 @@ gnomemeeting_tray_hack (gpointer data)
 
   gw = GnomeMeeting::Process ()->GetMainWindow ();
   
-  if (!gnomemeeting_tray_is_visible (gw->docklet)) {
+  if (!gnomemeeting_tray_is_embedded (gw->docklet)) {
 
     gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Notification area not detected"), _("You have chosen to start GnomeMeeting hidden, however the notification area is not present in your panel, GnomeMeeting can thus not start hidden."));
     gnomemeeting_window_show (gm);
@@ -102,8 +102,6 @@ GnomeMeeting::GnomeMeeting ()
   endpoint = NULL;
   url_handler = NULL;
 
-  client = gconf_client_get_default ();
-
   /* Init the different structures */
   gw = new GmWindow ();
   pw = new GmPrefWindow ();
@@ -115,7 +113,7 @@ GnomeMeeting::GnomeMeeting ()
 
   memset ((void *) rtp, 0, sizeof (struct _GmRtpData));
   gw->docklet = gw->ldap_window = gw->pref_window = gw->calls_history_window =
-    gw->splash_win = gw->incoming_call_popup = gw->history_window =
+    gw->splash_win = gw->incoming_call_popup = gw->log_window =
     gw->audio_transmission_popup = gw->audio_reception_popup =
 #ifndef DISABLE_GNOME
     gw->druid_window =
@@ -149,8 +147,8 @@ GnomeMeeting::~GnomeMeeting()
   
   if (gw->pref_window)
     gtk_widget_destroy (gw->pref_window);
-  if (gw->history_window)
-    gtk_widget_destroy (gw->history_window);
+  if (gw->log_window)
+    gtk_widget_destroy (gw->log_window);
   if (gw->calls_history_window)
     gtk_widget_destroy (gw->calls_history_window);
   if (gm)
@@ -185,8 +183,7 @@ GnomeMeeting::Connect()
  if (endpoint->GetCallingState () == 3) {
 
     gnomemeeting_threads_enter ();
-    gnomemeeting_log_insert (gw->history_text_view,
-			     _("Answering incoming call"));
+    gnomemeeting_log_insert (_("Answering incoming call"));
     connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 1);
     gnomemeeting_threads_leave ();
 
@@ -235,8 +232,7 @@ GnomeMeeting::Disconnect (H323Connection::CallEndReason reason)
   if (endpoint->GetCallingState () == 1) {
 
     gnomemeeting_threads_enter ();
-    gnomemeeting_log_insert (gw->history_text_view,
-			     _("Trying to stop calling"));
+    gnomemeeting_log_insert (_("Trying to stop calling"));
     gnomemeeting_threads_leave ();
 
     endpoint->ClearCall (endpoint->GetCurrentCallToken (), reason);
@@ -247,8 +243,7 @@ GnomeMeeting::Disconnect (H323Connection::CallEndReason reason)
     if (endpoint->GetCallingState () == 2) {
 
       gnomemeeting_threads_enter ();	
-      gnomemeeting_log_insert (gw->history_text_view,
-			       _("Stopping current call"));
+      gnomemeeting_log_insert (_("Stopping current call"));
       connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 
 				    0);
       gnomemeeting_threads_leave ();
@@ -258,8 +253,7 @@ GnomeMeeting::Disconnect (H323Connection::CallEndReason reason)
     else if (endpoint->GetCallingState () == 3) {
 
       gnomemeeting_threads_enter ();
-      gnomemeeting_log_insert (gw->history_text_view,
-			       _("Refusing Incoming call"));
+      gnomemeeting_log_insert (_("Refusing Incoming call"));
       connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 
 				    0);
       gnomemeeting_threads_leave ();
@@ -294,8 +288,6 @@ GnomeMeeting::DetectDevices ()
   audio_plugin = gconf_get_string (AUDIO_DEVICES_KEY "plugin");
   video_plugin = gconf_get_string (VIDEO_DEVICES_KEY "plugin");
  
-  if (!audio_plugin || !video_plugin)
-    return FALSE;
 
   /* Detect the devices */
   gnomemeeting_sound_daemons_suspend ();
@@ -305,8 +297,8 @@ GnomeMeeting::DetectDevices ()
   gw->video_managers = PVideoInputDevice::GetDriverNames ();
 
   /* Detect the devices */
-  gw->video_devices = PVideoInputDevice::GetDriversDeviceNames (video_plugin);
-
+   gw->video_devices = PVideoInputDevice::GetDriversDeviceNames (video_plugin);
+ 
   if (PString ("Quicknet") == audio_plugin) {
 
     gw->audio_recorder_devices = OpalIxJDevice::GetDeviceNames ();
@@ -419,7 +411,6 @@ void GnomeMeeting::Main ()
 
 void GnomeMeeting::BuildGUI ()
 {
-  gchar *msg = NULL;
   bool show_splash = TRUE;
 
   
@@ -428,7 +419,7 @@ void GnomeMeeting::BuildGUI ()
   g_signal_connect (G_OBJECT (gw->splash_win), "delete_event",
 		    G_CALLBACK (gtk_widget_hide_on_delete), 0);
 
-  show_splash = gconf_client_get_bool (client, USER_INTERFACE_KEY "show_splash_screen", 0);
+  show_splash = gconf_get_bool (USER_INTERFACE_KEY "show_splash_screen");
   if (show_splash) 
   {
     /* We show the splash screen */
@@ -442,7 +433,7 @@ void GnomeMeeting::BuildGUI ()
   /* Build the GUI */
   gnomemeeting_stock_icons_init ();
   gw->tips = gtk_tooltips_new ();
-  gw->history_window = gnomemeeting_history_window_new ();
+  gw->log_window = gnomemeeting_log_window_new ();
   gw->calls_history_window = gnomemeeting_calls_history_window_new (chw);
   gw->pc_to_phone_window = gnomemeeting_pc_to_phone_window_new ();  
   gw->pref_window = gnomemeeting_pref_window_new (pw);  
@@ -452,13 +443,14 @@ void GnomeMeeting::BuildGUI ()
 #endif
 #ifndef WIN32
   gw->docklet = gnomemeeting_init_tray ();
+  gw->tray_popup_menu = gnomemeeting_tray_init_menu (gw->docklet);
 #endif
   gnomemeeting_main_window_new (gw);
 
 
 #ifndef DISABLE_GNOME
- if (gconf_client_get_int (client, GENERAL_KEY "version", NULL) 
-      < 100 * MAJOR_VERSION + MINOR_VERSION) {
+ if (gconf_get_int (GENERAL_KEY "version") 
+      < 1000 * MAJOR_VERSION + 10 * MINOR_VERSION + BUILD_NUMBER) {
 
    gtk_widget_show_all (GTK_WIDGET (gw->druid_window));
   }
@@ -477,8 +469,6 @@ void GnomeMeeting::BuildGUI ()
   }
 #endif
 
- gconf_client_set_int (client, GENERAL_KEY "version", 
-		       100 * MAJOR_VERSION + MINOR_VERSION, NULL);
  
   /* Destroy the splash */
  if (gw->splash_win) {
@@ -489,10 +479,9 @@ void GnomeMeeting::BuildGUI ()
 
   
   /* GM is started */
-  msg = g_strdup_printf (_("Started GnomeMeeting V%d.%d for %s\n"), 
-			 MAJOR_VERSION, MINOR_VERSION, g_get_user_name ());
-  gnomemeeting_log_insert (gw->history_text_view, msg);
-  g_free (msg);
+  gnomemeeting_log_insert (_("Started GnomeMeeting V%d.%d.%d for %s\n"), 
+			   MAJOR_VERSION, MINOR_VERSION, BUILD_NUMBER,
+                           g_get_user_name ());
 }
 
 
