@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2003 Damien Sandras
+ * Copyright (C) 2000-2004 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
  *                         ils.cpp  -  description
  *                         -----------------------
  *   begin                : Sun Sep 23 2001
- *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   copyright            : (C) 2000-2004 by Damien Sandras
  *   description          : The ILS class thread.
  *
  */
@@ -42,6 +42,8 @@
 #include "gnomemeeting.h"
 #include "ldap_window.h"
 #include "misc.h"
+#include "tools.h"
+#include "gconf_widgets_extensions.h"
 #include "stock-icons.h"
 #include "dialog.h"
 
@@ -68,7 +70,6 @@ GMILSClient::GMILSClient ()
 
   operation = ILS_NONE;
 
-  client = gconf_client_get_default ();
 }
 
 
@@ -85,30 +86,22 @@ BOOL GMILSClient::CheckFieldsConfig (BOOL registering)
   bool no_error = TRUE;
 
   gnomemeeting_threads_enter ();
-  firstname =  
-    gconf_client_get_string (GCONF_CLIENT (client), 
-			     PERSONAL_DATA_KEY "firstname",
-			     NULL);
-  surname =  
-    gconf_client_get_string (GCONF_CLIENT (client),
-			     PERSONAL_DATA_KEY "lastname",
-			     NULL);
-  mail =  
-    gconf_client_get_string (GCONF_CLIENT (client),
-			     PERSONAL_DATA_KEY "mail",
-			     NULL);
+  firstname = gconf_get_string (PERSONAL_DATA_KEY "firstname");
+  surname = gconf_get_string (PERSONAL_DATA_KEY "lastname");
+  mail = gconf_get_string (PERSONAL_DATA_KEY "mail");
   gnomemeeting_threads_leave ();
 
 
   if (registering) {
+    
+    if ((!firstname || PString (firstname).Trim ().IsEmpty ())
+        || (!mail || PString (mail).Trim ().IsEmpty () 
+            || PString (mail).Find ("@") == P_MAX_INDEX)) {
 
-    if ((firstname == NULL) || (!strcmp (firstname, ""))
-	|| (mail == NULL) || (!strcmp (mail, ""))) {
-      
       /* No need to display that for unregistering */
       gnomemeeting_threads_enter ();
       gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Invalid parameters"), _("Please provide your first name and e-mail in the Personal Data section in order to be able to register to the user directory."));
-      gconf_client_set_bool (client, LDAP_KEY "enable_registering", FALSE, NULL);
+      gconf_set_bool (LDAP_KEY "enable_registering", FALSE);
       gnomemeeting_threads_leave ();
       
       no_error = FALSE;
@@ -133,8 +126,7 @@ BOOL GMILSClient::CheckServerConfig ()
 
 
   gnomemeeting_threads_enter ();
-  ldap_server =  
-    gconf_client_get_string (GCONF_CLIENT (client), LDAP_KEY "server", 0);
+  ldap_server = gconf_get_string (LDAP_KEY "server");
   gnomemeeting_threads_leave ();
 
 
@@ -212,9 +204,7 @@ void GMILSClient::ILSOperation (Operation operation)
   if (CheckServerConfig ()) {
 
     gnomemeeting_threads_enter ();
-    ldap_server =  
-      gconf_client_get_string (GCONF_CLIENT (client), LDAP_KEY "server", 
-			       NULL);
+    ldap_server = gconf_get_string (LDAP_KEY "server");
     gnomemeeting_threads_leave ();
 
 
@@ -234,7 +224,7 @@ void GMILSClient::ILSOperation (Operation operation)
 				&cred, &method))) {
 
       gnomemeeting_threads_enter ();      
-      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Bad information"), _("Bad ldap information from XML file: %s."), pferrtostring (rc));
+      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Bad information"), _("Bad LDAP information from XML file: %s."), pferrtostring (rc));
       gnomemeeting_threads_leave ();
 
       no_error = FALSE;
@@ -243,7 +233,7 @@ void GMILSClient::ILSOperation (Operation operation)
     else if (!(ldap = ldap_init (ldap_server, 389))) {
       
       gnomemeeting_threads_enter ();
-      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Cannot contact the users directory"), _("Failed to contact the users directory %s:%d. The directory is probably currently overloaded, try again later."), ldap_server, "389");
+      gnomemeeting_error_dialog (GTK_WINDOW (gm), _("Cannot contact the users directory"), _("Failed to contact the users directory %s:%d. The directory is probably currently overloaded, please try again later."), ldap_server, "389");
       gnomemeeting_threads_leave ();
 
       no_error = FALSE;
@@ -293,7 +283,7 @@ void GMILSClient::ILSOperation (Operation operation)
 	  msg = g_strdup_printf (_("Unregistered from the users directory %s."), ldap_server);	
 
 	gnomemeeting_threads_enter ();
-	gnomemeeting_log_insert (gw->history_text_view, msg);
+	gnomemeeting_log_insert (msg);
 	g_free (msg);
 	gnomemeeting_threads_leave ();
       }
@@ -312,9 +302,9 @@ void GMILSClient::ILSOperation (Operation operation)
   if (!no_error) {
 
     gnomemeeting_threads_enter ();
-    msg = g_strdup_printf (_("Error while registering to %s."),
+    msg = g_strdup_printf (_("Error while registering to %s"),
 			   ldap_server);
-    gnomemeeting_log_insert (gw->history_text_view, msg);
+    gnomemeeting_log_insert (msg);
     gnomemeeting_statusbar_flash (gw->statusbar, msg);
     g_free (msg);
     gnomemeeting_threads_leave ();
@@ -408,75 +398,53 @@ xmlEntityPtr xdap_getentity (void *ctx, const xmlChar * name)
   unsigned long int sip = 0;
 
   gnomemeeting_threads_enter ();
-  GConfClient *client = gconf_client_get_default ();
 
+  firstname = gconf_get_string (PERSONAL_DATA_KEY "firstname");
+  surname = gconf_get_string (PERSONAL_DATA_KEY "lastname");
 
-  firstname = 
-    gconf_client_get_string (GCONF_CLIENT (client),
-			     PERSONAL_DATA_KEY "firstname", 
-			     NULL);
-
-  surname =  
-    gconf_client_get_string (GCONF_CLIENT (client),
-			     PERSONAL_DATA_KEY "lastname", 
-			     NULL);
   if (!surname || !strcmp (surname, ""))
     surname = g_strdup ("-");
   tmp = g_strdup_printf ("%.65s", surname);
   g_free (surname);
   surname = tmp;
   
-  mail =  
-    gconf_client_get_string (GCONF_CLIENT (client),
-			     PERSONAL_DATA_KEY "mail", 
-			     NULL);
+  mail = gconf_get_string (PERSONAL_DATA_KEY "mail");
   tmp = g_strdup_printf ("%.65s", mail);
   g_free (mail);
   mail = tmp;
 
-  comment =  
-    gconf_client_get_string (GCONF_CLIENT (client),
-			     PERSONAL_DATA_KEY "comment", 
-			     NULL);
+  comment = gconf_get_string (PERSONAL_DATA_KEY "comment");
   if (!comment || !strcmp (comment, ""))
     comment = g_strdup ("-");
   tmp = g_strdup_printf ("%.65s", comment);
   g_free (comment);
   comment = tmp;
 
-  location =  
-    gconf_client_get_string (GCONF_CLIENT (client),
-			     PERSONAL_DATA_KEY "location", 
-			     NULL);
+  location = gconf_get_string (PERSONAL_DATA_KEY "location");
   if (!location || !strcmp (location, ""))
     location = g_strdup ("-");
   tmp = g_strdup_printf ("%.65s", location);
   g_free (location);
   location = tmp;
 
-  port = 
-    g_strdup_printf ("%d", 
-		     gconf_client_get_int (GCONF_CLIENT (client),
-					   PORTS_KEY "listen_port",
-					   NULL));
+  port = g_strdup_printf ("%d", gconf_get_int (PORTS_KEY "listen_port"));
 
   version =  g_strdup_printf ("%u", MAJOR_VERSION << 24 | 
 			            MINOR_VERSION << 16 |
 			            BUILD_NUMBER);
 
   if ((GnomeMeeting::Process ()->Endpoint ()->GetCallingState () != GMH323EndPoint::Standby)
-      || (gconf_client_get_int (client, CALL_OPTIONS_KEY "incoming_call_mode", 
-				 NULL)) == BUSY)
+      || (gconf_get_int (CALL_OPTIONS_KEY "incoming_call_mode") == BUSY))
     busy = g_strdup ("1");
   else
     busy = g_strdup ("0");
 
-  if (gconf_client_get_bool (client, LDAP_KEY "show_details", NULL))
+  if (gconf_get_bool (LDAP_KEY "show_details"))
     sflags = g_strdup ("1");
   else
     sflags = g_strdup ("0");
   
-  if (gconf_client_get_bool (client, VIDEO_CODECS_KEY "enable_video_transmission", NULL))
+  if (gconf_get_bool (VIDEO_CODECS_KEY "enable_video_transmission"))
     ilsa32964638 = g_strdup ("1");
   else
     ilsa32964638 = g_strdup ("0");
@@ -671,7 +639,7 @@ void GMILSBrowser::Main ()
       
       gnomemeeting_threads_enter ();
       gnomemeeting_statusbar_push (lwp->statusbar,
-				   _("Failed to contact LDAP server %s:%d."),
+				   _("Failed to contact LDAP server %s:%d"),
 				   ldap_server, "389");
       gnomemeeting_threads_leave ();
       
@@ -684,7 +652,7 @@ void GMILSBrowser::Main ()
      
       gnomemeeting_threads_enter ();
       gnomemeeting_statusbar_push (lwp->statusbar,
-				   _("Failed to set time limit on LDAP operations."));
+				   _("Failed to set time limit on LDAP operations"));
       gnomemeeting_threads_leave ();
     
       no_error = FALSE;  
@@ -694,7 +662,7 @@ void GMILSBrowser::Main ()
 				LDAP_AUTH_SIMPLE))) {
       
       gnomemeeting_threads_enter ();
-      gnomemeeting_statusbar_push (lwp->statusbar, _("Failed to contact to LDAP server %s: %s."), ldap_server, ldap_err2string (rc));
+      gnomemeeting_statusbar_push (lwp->statusbar, _("Failed to contact to LDAP server %s: %s"), ldap_server, ldap_err2string (rc));
       gnomemeeting_threads_leave ();
     
       no_error = FALSE;
@@ -704,7 +672,7 @@ void GMILSBrowser::Main ()
     
       gnomemeeting_threads_enter ();        
       gnomemeeting_statusbar_push (lwp->statusbar,
-				   _("Fetching online users list from %s."),
+				   _("Fetching online users list from %s"),
 				   ldap_server);
       gnomemeeting_threads_leave ();
       
@@ -722,7 +690,7 @@ void GMILSBrowser::Main ()
       gnomemeeting_threads_enter ();
       if (rc != 0)
 	if (rc != LDAP_SERVER_DOWN) 
-	  gnomemeeting_statusbar_push (lwp->statusbar, _("Could not fetch online users list."));
+	  gnomemeeting_statusbar_push (lwp->statusbar, _("Could not fetch online users list"));
       gnomemeeting_threads_leave ();
   
       if (rc == LDAP_SERVER_DOWN)
@@ -738,7 +706,7 @@ void GMILSBrowser::Main ()
     
     gnomemeeting_threads_enter ();
     gnomemeeting_statusbar_push (lwp->statusbar,
-				 _("Connection to %s lost..."),
+				 _("Connection to %s lost"),
 				 ldap_server);
     gnomemeeting_threads_leave ();
   }
@@ -884,7 +852,7 @@ void GMILSBrowser::Main ()
 	 
 	status_icon =
 	  gtk_widget_render_icon (lwp->tree_view, 
-				  GM_STOCK_STATUS_OCCUPIED,
+				  GM_STOCK_STATUS_BUSY,
 				  GTK_ICON_SIZE_MENU, NULL);
 	color = g_strdup ("#8a8a8a");
       }        
@@ -955,12 +923,12 @@ void GMILSBrowser::Main ()
 
     if (num_users && num_users [1]) {
 
-      gnomemeeting_statusbar_push (lwp->statusbar, _("Search completed: %d user(s) listed on a total of %d user(s) from %s."), users_nbr, PMAX (atoi (num_users [1]), users_nbr), ldap_server);
+      gnomemeeting_statusbar_push (lwp->statusbar, _("Search completed: %d user(s) listed on a total of %d user(s) from %s"), users_nbr, PMAX (atoi (num_users [1]), users_nbr), ldap_server);
 
       g_strfreev (num_users);
     }
     else
-      gnomemeeting_statusbar_push (lwp->statusbar, _("Search completed: %d user(s) found on %s."), users_nbr, ldap_server);
+      gnomemeeting_statusbar_push (lwp->statusbar, _("Search completed: %d user(s) found on %s"), users_nbr, ldap_server);
 
     gnomemeeting_threads_leave ();
   }
