@@ -1,6 +1,6 @@
 
 /* GnomeMeeting -- A Video-Conferencing application
- * Copyright (C) 2000-2003 Damien Sandras
+ * Copyright (C) 2000-2004 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
  *                         main_window.cpp  -  description
  *                         -------------------------------
  *   begin                : Mon Mar 26 2001
- *   copyright            : (C) 2000-2003 by Damien Sandras
+ *   copyright            : (C) 2000-2004 by Damien Sandras
  *   description          : This file contains all the functions needed to
  *                          build the main window.
  */
@@ -47,6 +47,7 @@
 #include "callbacks.h"
 #include "tray.h"
 #include "lid.h"
+#include "sound_handling.h"
 
 #include "dialog.h"
 #include "gmentrydialog.h"
@@ -571,12 +572,10 @@ static void
 main_notebook_page_changed (GtkNotebook *notebook, GtkNotebookPage *page,
 			    gint page_num, gpointer user_data) 
 {
-  GConfClient *client = gconf_client_get_default ();
-
   GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
-  gconf_client_set_int (client, USER_INTERFACE_KEY "main_window/control_panel_section",
-			gtk_notebook_get_current_page (GTK_NOTEBOOK (gw->main_notebook)), 0);
+  gconf_set_int (USER_INTERFACE_KEY "main_window/control_panel_section",
+		 gtk_notebook_get_current_page (GTK_NOTEBOOK (gw->main_notebook)));
 }
 
 
@@ -591,6 +590,12 @@ audio_volume_changed (GtkAdjustment *adjustment, gpointer data)
 {
   GmWindow *gw = NULL;
   GMH323EndPoint *ep = NULL;
+  
+  H323Connection *con = NULL;
+  H323Codec *raw_codec = NULL;
+  H323Channel *channel = NULL;
+
+  PSoundChannel *sound_channel = NULL;
 
   unsigned int play_vol =  0, rec_vol = 0;
 
@@ -601,7 +606,33 @@ audio_volume_changed (GtkAdjustment *adjustment, gpointer data)
   rec_vol = (unsigned int) (GTK_ADJUSTMENT (gw->adj_rec)->value);
 
   gdk_threads_leave ();
-  ep->SetDeviceVolume (play_vol, rec_vol);  
+ 
+  con = ep->FindConnectionWithLock (ep->GetCurrentCallToken ());
+
+  if (con) {
+
+    for (int cpt = 0 ; cpt < 2 ; cpt++) {
+
+      channel = 
+        con->FindChannel (RTP_Session::DefaultAudioSessionID, (cpt == 0));         
+      if (channel) {
+
+        raw_codec = channel->GetCodec();
+
+        if (raw_codec) {
+
+          sound_channel = (PSoundChannel *) raw_codec->GetRawDataChannel ();
+
+          if (sound_channel)
+            ep->SetDeviceVolume (sound_channel, 
+                                 (cpt == 1), 
+                                 (cpt == 1) ? rec_vol : play_vol);
+        }
+      }
+    }
+    con->Unlock ();
+  }
+
   gdk_threads_enter ();
 }
 
@@ -764,12 +795,12 @@ gm_quit_callback (GtkWidget *widget, GdkEvent *event,
   gboolean b;
   GmWindow *gw = GnomeMeeting::Process ()->GetMainWindow ();
 
-  b = gnomemeeting_tray_is_visible (gw->docklet);
+  b = gnomemeeting_tray_is_embedded (gw->docklet);
 
   if (!b)
     quit_callback (NULL, data);
   else 
-    gtk_widget_hide (GTK_WIDGET (gm));
+    gnomemeeting_window_hide (GTK_WIDGET (gm));
 
   return (TRUE);
 }  
@@ -814,7 +845,7 @@ void gnomemeeting_dialpad_event (const char d)
   if (gw->transfer_call_popup)
     gm_entry_dialog_set_text (GM_ENTRY_DIALOG (gw->transfer_call_popup),
 			      new_url);
-  else 
+  else if (endpoint->GetCallingState () == GMH323EndPoint::Standby)
     gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (gw->combo)->entry), new_url);
 
   if (dtmf == '#' && gw->transfer_call_popup)
@@ -917,8 +948,6 @@ gnomemeeting_main_window_update_sensitivity (BOOL is_video,
   
   gtk_widget_set_sensitive (GTK_WIDGET (button), is_transmitting);
   gtk_widget_set_sensitive (GTK_WIDGET (frame), is_transmitting);
-  GTK_TOGGLE_BUTTON (button)->active = is_transmitting;
-  gtk_widget_queue_draw (button);
 }
 
 
@@ -943,14 +972,12 @@ gnomemeeting_main_window_new (GmWindow *gw)
   int main_notebook_section = 0;
 
   GmTextChat *chat = NULL;
-  GConfClient *client = NULL;
 
   static GtkTargetEntry dnd_targets [] =
   {
     {"text/plain", GTK_TARGET_SAME_APP, 0}
   };
 
-  client = gconf_client_get_default ();
   chat = GnomeMeeting::Process ()->GetTextChat ();
   
   accel = gtk_accel_group_new ();
@@ -980,8 +1007,7 @@ gnomemeeting_main_window_new (GmWindow *gw)
   gtk_box_pack_start (GTK_BOX (hbox), gw->statusbar, 
 		      TRUE, TRUE, 0);
 
-  if (gconf_client_get_bool (client, 
-			     USER_INTERFACE_KEY "main_window/show_status_bar", 0))
+  if (gconf_get_bool (USER_INTERFACE_KEY "main_window/show_status_bar"))
     gtk_widget_show (GTK_WIDGET (gw->statusbar));
   else
     gtk_widget_hide (GTK_WIDGET (gw->statusbar));
@@ -1056,7 +1082,7 @@ gnomemeeting_main_window_new (GmWindow *gw)
 		    6, 6); 
 
   main_notebook_section = 
-    gconf_client_get_int (client, USER_INTERFACE_KEY "main_window/control_panel_section", 0);
+    gconf_get_int (USER_INTERFACE_KEY "main_window/control_panel_section");
 
   if (main_notebook_section != GM_MAIN_NOTEBOOK_HIDDEN) {
 
@@ -1141,8 +1167,7 @@ gnomemeeting_main_window_new (GmWindow *gw)
  		    (GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
  		    (GtkAttachOptions) (GTK_FILL | GTK_EXPAND),
  		    6, 6);
-  if (gconf_client_get_bool 
-      (client, USER_INTERFACE_KEY "main_window/show_chat_window", 0))
+  if (gconf_get_bool (USER_INTERFACE_KEY "main_window/show_chat_window"))
     gtk_widget_show_all (GTK_WIDGET (gw->chat_window));
   
   gtk_widget_set_size_request (GTK_WIDGET (gw->main_notebook),
@@ -1562,6 +1587,12 @@ int main (int argc, char ** argv, char ** envp)
     };
 
 
+  /* Select the Mic as default source for OSS. Will be removed when
+   * ALSA will be everywhere 
+   */
+  gnomemeeting_mixers_mic_select ();
+
+
   /* GnomeMeeting Initialisation */
   gnome_program_init ("gnomemeeting", VERSION,
 		      LIBGNOMEUI_MODULE, argc, argv,
@@ -1604,6 +1635,12 @@ int main (int argc, char ** argv, char ** envp)
   /* GnomeMeeting main initialisation */
   static GnomeMeeting instance;
 
+  /* Debug */
+  if (debug_level != 0)
+    PTrace::Initialise (PMAX (PMIN (4, debug_level), 0), NULL,
+			PTrace::Timestamp | PTrace::Thread
+			| PTrace::Blocks | PTrace::DateAndTime);
+
   
   /* Detect the devices, exit if it fails */
   if (!GnomeMeeting::Process ()->DetectDevices ()) {
@@ -1646,14 +1683,6 @@ int main (int argc, char ** argv, char ** envp)
   }
 
   
-  /* Debug */
-  if (debug_level != 0)
-    PTrace::Initialise (PMAX (PMIN (4, debug_level), 0));
-  /*			"gnomemeeting-debug.log",
-			PTrace::Timestamp | PTrace::Thread
-			| PTrace::Blocks | PTrace::DateAndTime);
-  */
-
   /* Call the given host if needed */
   if (url) {
 
